import numpy as np # type: ignore
import pandas as pd # type: ignore
import cv2 # type: ignore
import matplotlib.pyplot as plt # type: ignore
import os # type: ignore
import matplotlib.image as mpimg # type: ignore
from sklearn.preprocessing import MinMaxScaler # type: ignore
from sklearn.model_selection import train_test_split # type: ignore
from sklearn.preprocessing import LabelEncoder # type: ignore
from PIL import Image # type: ignore